/*
 * ============LICENSE_START=======================================================
 * PNF-REGISTRATION-HANDLER
 * ================================================================================
 * Copyright (C) 2018-2019 NOKIA Intellectual Property. All rights reserved.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ============LICENSE_END=========================================================
 */

package org.onap.dcaegen2.services.prh.configuration;

import org.onap.dcaegen2.services.sdk.rest.services.aai.client.config.AaiClientConfiguration;
import org.onap.dcaegen2.services.sdk.rest.services.dmaap.client.model.MessageRouterPublishRequest;
import org.onap.dcaegen2.services.sdk.rest.services.dmaap.client.model.MessageRouterSubscribeRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.util.StreamUtils;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.nio.charset.Charset;

/**
 * @author <a href="mailto:przemyslaw.wasala@nokia.com">Przemysław Wąsala</a> on 4/9/18
 */
@Configuration
@EnableConfigurationProperties
@ConfigurationProperties("app")
public abstract class PrhAppConfig implements Config {
    private static final Logger LOGGER = LoggerFactory.getLogger(PrhAppConfig.class);

    AaiClientConfiguration aaiClientConfiguration;

    MessageRouterSubscribeRequest messageRouterSubscribeRequest;

    MessageRouterPublishRequest messageRouterPublishRequest;

    MessageRouterPublishRequest messageRouterUpdatePublishRequest;

    @Value("classpath:git_info.json")
    private Resource gitInfo;

    @PostConstruct
    private void printGitInfo() throws IOException {
        LOGGER.debug("Git info={}", StreamUtils.copyToString(gitInfo.getInputStream(), Charset.defaultCharset()));
    }

    @Override
    public Resource getGitInfo() {
        return gitInfo;
    }

    @Override
    public MessageRouterSubscribeRequest getMessageRouterSubscribeRequest() {
        return messageRouterSubscribeRequest;
    }

    @Override
    public AaiClientConfiguration getAaiClientConfiguration() {
        return aaiClientConfiguration;
    }

    @Override
    public MessageRouterPublishRequest getMessageRouterPublishRequest() {
        return messageRouterPublishRequest;
    }

    @Override
    public MessageRouterPublishRequest getMessageRouterUpdatePublishRequest() {
        return messageRouterUpdatePublishRequest;
    }
}
